@@ -239,7 +239,7 @@ command_thread_start (void *arg)
 	if (g_spawn_sync (NULL,
 			  (char **) data->argv->pdata,
 			  NULL,
-			  G_SPAWN_STDOUT_TO_DEV_NULL|G_SPAWN_STDERR_TO_DEV_NULL,
+			  0,
 			  NULL, NULL,
 			  NULL,
 			  NULL,
@@ -322,7 +322,7 @@ launch_command (NautilusBurnDrive *drive,
 
 		if (data->error)
 			g_propagate_error (error, data->error);
-		else if (!command_ok)
+		else
 			g_set_error (error,
 				     BRASERO_BURN_ERROR,
 				     BRASERO_BURN_ERROR_GENERAL,
@@ -338,16 +338,6 @@ launch_command (NautilusBurnDrive *drive,
 	return command_ok;
 }
 
-/**
- * nautilus_burn_drive_unmount:
- * @drive: #NautilusBurnDrive
- *
- * Unmount the media in a #NautilusBurnDrive.
- *
- * Return value: %TRUE if the media was sucessfully unmounted, %FALSE otherwise.
- *
- * Since: 2.10
- **/
 gboolean
 NCB_DRIVE_UNMOUNT (NautilusBurnDrive *drive,
 		   GError **error)
@@ -362,39 +352,6 @@ NCB_DRIVE_MOUNT (NautilusBurnDrive *drive,
 	return launch_command (drive, TRUE, error);
 }
 
-struct _BraseroMountData {
-	gboolean mounted_by_us;
-	NautilusBurnDrive *drive;
-
-	BraseroMountCallback callback_func;
-	gpointer callback_data;
-
-	guint num_attempts;
-	gint mount_timeout;
-};
-typedef struct _BraseroMountData BraseroMountData;
-
-#define MAX_MOUNT_ATTEMPS 10
-#define MOUNT_TIMEOUT 500
-
-static void
-NCB_DRIVE_GET_MOUNT_POINT_RESULT (BraseroMountData *data,
-				  gchar *mount_point,
-				  GError *error)
-{
-	data->callback_func (data->drive,
-			     mount_point,
-			     data->mounted_by_us,
-			     error,
-			     data->callback_data);
-
-	if (error)
-		g_error_free (error);
-
-	nautilus_burn_drive_unref (data->drive);
-	g_free (data);
-}
-
 static GnomeVFSDrive *
 NCB_DRIVE_GET_VFS_DRIVE (NautilusBurnDrive *drive)
 {
@@ -436,28 +393,17 @@ NCB_DRIVE_GET_VFS_DRIVE (NautilusBurnDrive *drive)
 	return NULL;
 }
 
-static gboolean
-NCB_VOLUME_GET_MOUNT_POINT_CB (gpointer callback_data)
+gchar *
+NCB_VOLUME_GET_MOUNT_POINT (NautilusBurnDrive *drive,
+			    GError **error)
 {
-	BraseroMountData *data = callback_data;
 	GnomeVFSDrive *vfsdrive = NULL;
 	gchar *mount_point = NULL;
 	gchar *local_path = NULL;
 	GList *iter, *volumes;
-	GError *error = NULL;
-
-	if (!nautilus_burn_drive_is_mounted (data->drive)) {
-		/* try to mount the disc */
-		if (!NCB_DRIVE_MOUNT (data->drive, &error)) {
-			NCB_DRIVE_GET_MOUNT_POINT_RESULT (data, NULL, error);
-			return FALSE;
-		}
-
-		data->mounted_by_us = TRUE;
-	}
 
 	/* get the uri for the mount point */
-	vfsdrive = NCB_DRIVE_GET_VFS_DRIVE (data->drive);
+	vfsdrive = NCB_DRIVE_GET_VFS_DRIVE (drive);
 	volumes = gnome_vfs_drive_get_mounted_volumes (vfsdrive);
 	gnome_vfs_drive_unref (vfsdrive);
 
@@ -471,91 +417,157 @@ NCB_VOLUME_GET_MOUNT_POINT_CB (gpointer callback_data)
 	}
 	gnome_vfs_drive_volume_list_free (volumes);
 
-	if (!mount_point) {
-		/* retry if we couldn't get the mount point this time */
-		data->num_attempts ++;
-		if (data->num_attempts < MAX_MOUNT_ATTEMPS)
-			return TRUE;
-	}
-
 	if (!mount_point || strncmp (mount_point, "file://", 7)) {
 		/* mount point won't be usable */
-		if (mount_point)
+		if (mount_point) {
 			g_free (mount_point);
+			mount_point = NULL;
+		}
 
-		error = g_error_new (BRASERO_BURN_ERROR,
-				     BRASERO_BURN_ERROR_GENERAL,
-				     _("the disc mount point could not be retrieved."));
+		g_set_error (error,
+			     BRASERO_BURN_ERROR,
+			     BRASERO_BURN_ERROR_GENERAL,
+			     _("the disc mount point could not be retrieved."));
 	}
 	else {
 		local_path = gnome_vfs_get_local_path_from_uri (mount_point);
 		g_free (mount_point);
 	}
 
-	NCB_DRIVE_GET_MOUNT_POINT_RESULT (data, local_path, error);
-	return FALSE;
+	return local_path;
 }
 
-static BraseroMountHandle *
-NCB_VOLUME_GET_MOUNT_POINT (NautilusBurnDrive *drive,
-			    BraseroMountCallback callback,
-			    gpointer callback_data)
+gboolean
+NCB_MEDIA_GET_LAST_DATA_TRACK_ADDRESS (NautilusBurnDrive *drive,
+				       gint64 *byte,
+				       gint64 *sector)
 {
-	BraseroMountData *data;
+	BraseroMedium *medium;
 
-	data = g_new0 (BraseroMountData,1);
+	if (!drive) {
+		if (byte)
+			*byte = -1;
+		if (sector)
+			*sector = -1;
+		return FALSE;
+	}
 
-	data->callback_func = callback;
-	data->callback_data = callback_data;
+	medium = g_object_get_data (G_OBJECT (drive), BRASERO_MEDIUM_KEY);
+	if (!medium) {
+		if (byte)
+			*byte = -1;
+		if (sector)
+			*sector = -1;
+		return FALSE;
+	}
 
-	data->drive = drive;
-	nautilus_burn_drive_ref (drive);
-
-	data->mount_timeout = g_timeout_add (MOUNT_TIMEOUT,
-					     NCB_VOLUME_GET_MOUNT_POINT_CB,
-					     data);
-
-	return (BraseroMountHandle *) data;
+	return brasero_medium_get_last_data_track_address (medium,
+							   byte,
+							   sector);
 }
 
-BraseroMountHandle *
-NCB_DRIVE_GET_MOUNT_POINT (NautilusBurnDrive *drive,
-			   BraseroMountCallback callback,
-			   gpointer callback_data)
+gboolean
+NCB_MEDIA_GET_LAST_DATA_TRACK_SPACE (NautilusBurnDrive *drive,
+				     gint64 *size,
+				     gint64 *blocks)
 {
-	BraseroMountHandle *handle;
+	BraseroMedium *medium;
 
-	handle = NCB_VOLUME_GET_MOUNT_POINT (drive,
-					     callback,
-					     callback_data);
+	if (!drive) {
+		if (size)
+			*size = -1;
+		if (blocks)
+			*blocks = -1;
+		return FALSE;
+	}
 
-	return handle;
+	medium = g_object_get_data (G_OBJECT (drive), BRASERO_MEDIUM_KEY);
+	if (!medium) {
+		if (size)
+			*size = -1;
+		if (blocks)
+			*blocks = -1;
+		return FALSE;
+	}
+
+	return brasero_medium_get_last_data_track_space (medium, size, blocks);
 }
 
-void
-NCB_DRIVE_GET_MOUNT_POINT_CANCEL (BraseroMountHandle handle)
-{
-	BraseroMountData *data = handle;
-
-	g_source_remove (data->mount_timeout);
-	nautilus_burn_drive_unref (data->drive);
-
-	g_free (data);
-}
-
-gint64
-NCB_MEDIA_GET_LAST_DATA_TRACK_ADDRESS (NautilusBurnDrive *drive)
+guint
+NCB_MEDIA_GET_TRACK_NUM (NautilusBurnDrive *drive)
 {
 	BraseroMedium *medium;
 
 	if (!drive)
-		return -1;
+		return 0;
 
 	medium = g_object_get_data (G_OBJECT (drive), BRASERO_MEDIUM_KEY);
 	if (!medium)
-		return -1;
+		return 0;
 
-	return brasero_medium_get_last_data_track_address (medium);
+	return brasero_medium_get_track_num (medium);
+}
+
+gboolean
+NCB_MEDIA_GET_TRACK_ADDRESS (NautilusBurnDrive *drive,
+			     guint num,
+			     gint64 *byte,
+			     gint64 *sector)
+{
+	BraseroMedium *medium;
+
+	if (!drive) {
+		if (byte)
+			*byte = -1;
+		if (sector)
+			*sector = -1;
+		return FALSE;
+	}
+
+	medium = g_object_get_data (G_OBJECT (drive), BRASERO_MEDIUM_KEY);
+	if (!medium) {
+		if (byte)
+			*byte = -1;
+		if (sector)
+			*sector = -1;
+		return FALSE;
+	}
+
+	return brasero_medium_get_track_address (medium,
+						 num,
+						 byte,
+						 sector);
+}
+
+gboolean
+NCB_MEDIA_GET_TRACK_SPACE (NautilusBurnDrive *drive,
+			   guint num,
+			   gint64 *size,
+			   gint64 *blocks)
+{
+	BraseroMedium *medium;
+
+	if (!drive) {
+		if (size)
+			*size = -1;
+		if (blocks)
+			*blocks = -1;
+		return FALSE;
+	}
+
+	medium = g_object_get_data (G_OBJECT (drive), BRASERO_MEDIUM_KEY);
+	if (!medium) {
+		if (size)
+			*size = -1;
+		if (blocks)
+			*blocks = -1;
+		return FALSE;
+	}
+
+	return brasero_medium_get_track_space (medium,
+					       num,
+					       size,
+					       blocks);
 }
 
 gint64
@@ -573,8 +585,8 @@ NCB_MEDIA_GET_NEXT_WRITABLE_ADDRESS (NautilusBurnDrive *drive)
 	return brasero_medium_get_next_writable_address (medium);
 }
 
-gint
-NCB_MEDIA_GET_MAX_WRITE_SPEED (NautilusBurnDrive *drive)
+gint64
+NCB_MEDIA_GET_MAX_WRITE_RATE (NautilusBurnDrive *drive)
 {
 	BraseroMedium *medium;
 
@@ -588,7 +600,7 @@ NCB_MEDIA_GET_MAX_WRITE_SPEED (NautilusBurnDrive *drive)
 	return brasero_medium_get_max_write_speed (medium);
 }
 
-BraseroMediumInfo
+BraseroMedia
 NCB_MEDIA_GET_STATUS (NautilusBurnDrive *drive)
 {
 	BraseroMedium *medium;
@@ -755,6 +767,7 @@ NCB_INIT (void)
 				   BRASERO_MEDIUM_KEY,
 				   medium);
 	}
+	g_list_free (list);
 
 	g_signal_connect (monitor,
 			  "media-added",
