@@ -37,9 +37,10 @@
 #include <totem-pl-parser.h>
 #endif
 
+#include "burn-debug.h"
 #include "brasero-vfs.h"
 #include "brasero-async-task-manager.h"
-#include "metadata.h"
+#include "brasero-metadata.h"
 
 static void brasero_vfs_class_init (BraseroVFSClass *klass);
 static void brasero_vfs_init (BraseroVFS *sp);
@@ -78,17 +79,19 @@ struct _BraseroVFSTaskCtx {
 };
 typedef struct _BraseroVFSTaskCtx BraseroVFSTaskCtx;
 
-struct _BraseroVFSMetaCtx {
-	BraseroVFSTaskCtx *ctx;
-	GnomeVFSFileInfo *info;
+struct _BraseroVFSMetadataTask {
+	gchar *uri;
+	GSList *ctx;
 	BraseroMetadata *metadata;
 };
-typedef struct _BraseroVFSMetaCtx BraseroVFSMetaCtx;
+typedef struct _BraseroVFSMetadataTask BraseroVFSMetadataTask;
 
 struct _BraseroInfoData {
 	GSList *results;
 	GSList *uris;
 	gint flags;
+
+	guint check_parent_sym:1;
 };
 typedef struct _BraseroInfoData BraseroInfoData;
 
@@ -103,6 +106,7 @@ typedef struct _BraseroLoadData BraseroLoadData;
 struct _BraseroMetadataData {
 	gint refcount;
 	gboolean recursive;
+	BraseroMetadataFlag flags;
 	GSList *results;
 };
 typedef struct _BraseroMetadataData BraseroMetadataData;
@@ -119,6 +123,8 @@ typedef struct _BraseroCountData BraseroCountData;
 
 struct _BraseroVFSPlaylistData {
 	gchar *uri;
+	gchar *title;
+
 	GList *uris;
 	TotemPlParser *parser;
 	TotemPlParserResult res;
@@ -137,11 +143,11 @@ typedef struct _BraseroInfoResult BraseroInfoResult;
 
 struct _BraseroMetadataResult {
 	GnomeVFSFileInfo *info;
-	BraseroMetadata *metadata;
-	gulong signal;
+	BraseroVFSMetadataTask *item;
 };
 typedef struct _BraseroMetadataResult BraseroMetadataResult;
 
+
 #define BRASERO_CTX_OWNER(ctx)			((ctx)->user_method->owner)
 #define BRASERO_CTX_USER_DATA(ctx)		((ctx)->user_data)
 
@@ -153,6 +159,7 @@ typedef struct _BraseroMetadataResult BraseroMetadataResult;
 #define BRASERO_CTX_INFO_CALLBACK(ctx)		((BraseroVFSInfoCallback) BRASERO_CTX_CALLBACK (ctx))
 #define BRASERO_CTX_COUNT_CALLBACK(ctx)		((BraseroVFSCountCallback) BRASERO_CTX_CALLBACK (ctx))
 #define BRASERO_CTX_META_CALLBACK(ctx)		((BraseroVFSMetaCallback) BRASERO_CTX_CALLBACK (ctx))
+#define BRASERO_CTX_PLAYLIST_CALLBACK(ctx)	((BraseroVFSPlaylistCallback) BRASERO_CTX_CALLBACK (ctx))
 
 #define BRASERO_CTX_INFO_CB(ctx, result, uri, info)				\
 	if (BRASERO_CTX_CALLBACK (ctx))						\
@@ -191,15 +198,6 @@ typedef struct _BraseroMetadataResult BraseroMetadataResult;
 						   metadata,			\
 						   BRASERO_CTX_USER_DATA (ctx))
 
-#define BRASERO_CTX_PLAYLIST_CB(ctx, result, uri, info, metadata)		\
-	if (BRASERO_CTX_CALLBACK (ctx))						\
-		(BRASERO_CTX_META_CALLBACK (ctx)) (BRASERO_CTX_SELF (ctx),	\
-						   BRASERO_CTX_OWNER (ctx),	\
-						   result,			\
-						   uri,				\
-						   info,			\
-						   metadata,			\
-						   BRASERO_CTX_USER_DATA (ctx))
 
 #define BRASERO_CTX_SET_TASK_METHOD(self, ctx, id) 				\
 	((ctx)->task_method = g_hash_table_lookup (self->priv->types, GINT_TO_POINTER (id)))
@@ -223,6 +221,7 @@ struct _BraseroVFSPrivate {
 #endif
 
 	/* used for metadata */
+	GSList *metadatas;
 	GSList *processing_meta;
 	GSList *unprocessed_meta;
 	gint meta_process_id;
@@ -235,9 +234,8 @@ struct _BraseroVFSPrivate {
 };
 
 /* so far one metadata at a time has shown to be the best for performance */
-#define MAX_CONCURENT_META 	1
-#define MAX_BUFFERED_META	8
-#define STOP_META_TIMEOUT	500
+#define MAX_CONCURENT_META 	2
+#define MAX_BUFFERED_META	20
 
 static GObjectClass *parent_class = NULL;
 static BraseroVFS *singleton = NULL;
@@ -306,6 +304,109 @@ brasero_vfs_get_default ()
 	return singleton;
 }
 
+/**
+ * This part deals with symlinks, that allows to get unique filenames by
+ * replacing any parent symlink by its target and check for recursive
+ * symlinks
+ */
+
+static gchar *
+brasero_vfs_check_for_parent_symlink (const gchar *escaped_uri)
+{
+	GnomeVFSFileInfo *info;
+	GnomeVFSURI *parent;
+    	gchar *uri;
+
+    	parent = gnome_vfs_uri_new (escaped_uri);
+  	info = gnome_vfs_file_info_new ();
+    	uri = gnome_vfs_uri_to_string (parent, GNOME_VFS_URI_HIDE_NONE);
+
+	while (gnome_vfs_uri_has_parent (parent)) {
+	    	GnomeVFSURI *tmp;
+		GnomeVFSResult result;
+
+		result = gnome_vfs_get_file_info_uri (parent,
+						      info,
+					              GNOME_VFS_FILE_INFO_FOLLOW_LINKS);
+
+		if (result != GNOME_VFS_OK)
+			/* we shouldn't reached this point but who knows */
+		    	break;
+
+		/* NOTE: no need to check for broken symlinks since
+		 * we wouldn't have reached this point otherwise */
+		if (GNOME_VFS_FILE_INFO_SYMLINK (info)) {
+		    	gchar *parent_uri;
+		    	gchar *new_root;
+			gchar *newuri;
+
+		    	parent_uri = gnome_vfs_uri_to_string (parent, GNOME_VFS_URI_HIDE_NONE);
+			new_root = gnome_vfs_make_uri_from_input (info->symlink_name);
+
+			newuri = g_strconcat (new_root,
+					      uri + strlen (parent_uri),
+					      NULL);
+
+		    	g_free (uri);
+		    	uri = newuri;	
+
+		    	gnome_vfs_uri_unref (parent);
+		    	g_free (parent_uri);
+
+		    	parent = gnome_vfs_uri_new (new_root);
+			g_free (new_root);
+		}
+
+		tmp = parent;
+		parent = gnome_vfs_uri_get_parent (parent);
+		gnome_vfs_uri_unref (tmp);
+
+		gnome_vfs_file_info_clear (info);
+	}
+	gnome_vfs_file_info_unref (info);
+	gnome_vfs_uri_unref (parent);
+
+	return uri;
+}
+
+static gboolean
+brasero_utils_get_symlink_target (const gchar *escaped_uri,
+				  GnomeVFSFileInfo *info,
+				  GnomeVFSFileInfoOptions flags)
+{
+	gint size;
+	GnomeVFSResult result;
+
+	result = gnome_vfs_get_file_info (escaped_uri,
+					  info,
+					  flags|
+					  GNOME_VFS_FILE_INFO_FOLLOW_LINKS);
+	if (result)
+		return FALSE;
+
+    	if (info->symlink_name) {
+		gchar *target;
+
+		target = gnome_vfs_make_uri_from_input (info->symlink_name);
+
+		g_free (info->symlink_name);
+		info->symlink_name = brasero_vfs_check_for_parent_symlink (target);
+		g_free (target);
+	}
+
+    	if (!info->symlink_name)
+		return FALSE;
+
+	/* we check for circular dependency here :
+	 * if the target is one of the parent of symlink */
+	size = strlen (info->symlink_name);
+	if (!strncmp (info->symlink_name, escaped_uri, size)
+	&& (*(escaped_uri + size) == '/' || *(escaped_uri + size) == '\0'))
+		return FALSE;
+	
+	return TRUE;
+}
+
 static void
 brasero_vfs_task_ctx_free (BraseroVFSTaskCtx *ctx, gboolean cancelled)
 {
@@ -371,7 +472,7 @@ brasero_vfs_info_destroy (GObject *obj, gpointer user_data, gboolean cancelled)
 	}
 
 	if (data->uris) {
-		g_slist_foreach (data->uris, (GFunc) gnome_vfs_uri_unref, NULL);
+		g_slist_foreach (data->uris, (GFunc) g_free, NULL);
 		g_slist_free (data->uris);
 	}
 
@@ -416,25 +517,60 @@ brasero_vfs_info_thread (BraseroAsyncTaskManager *manager,
 	data = BRASERO_CTX_TASK_DATA (ctx);
 	for (iter = data->uris; iter; iter = next) {
 		BraseroInfoResult *result;
+		GnomeVFSFileInfo *info;
 		GnomeVFSURI *vfsuri;
+		gchar *uri;
 
 		if (ctx->cancelled)
 			break;
 
 		next = iter->next;
-		vfsuri = iter->data;
-		data->uris = g_slist_remove (data->uris, vfsuri);
+		uri = iter->data;
+		data->uris = g_slist_remove (data->uris, uri);
 
 		result = g_new0 (BraseroInfoResult, 1);
 		data->results = g_slist_prepend (data->results, result);
 
-		result->info = gnome_vfs_file_info_new ();
-		result->uri = gnome_vfs_uri_to_string (vfsuri, GNOME_VFS_URI_HIDE_NONE);
-		result->result = gnome_vfs_get_file_info_uri (vfsuri,
-							      result->info,
-							      data->flags);
+		if (data->check_parent_sym) {
+			gchar *tmp;
 
+			/* If we want to make sure a directory is not added twice we have to make sure
+			 * that it doesn't have a symlink as parent otherwise "/home/Foo/Bar" with Foo
+			 * as a symlink pointing to /tmp would be seen as a different file from /tmp/Bar 
+			 * It would be much better if we could use the inode numbers provided by gnome_vfs
+			 * unfortunately they are guint64 and can't be used in hash tables as keys.
+			 * Therefore we check parents up to root to see if there are symlinks and if so
+			 * we get a path without symlinks in it. This is done only for local file */
+			tmp = uri;
+			uri = brasero_vfs_check_for_parent_symlink (uri);
+			g_free (tmp);
+		}
+
+		result->uri = uri;
+		info = gnome_vfs_file_info_new ();
+		vfsuri = gnome_vfs_uri_new (uri);
+		result->result = gnome_vfs_get_file_info_uri (vfsuri,
+							      info,
+							      data->flags);
 		gnome_vfs_uri_unref (vfsuri);
+
+		if (result->result != GNOME_VFS_OK) {
+			result->info = info;
+			continue;
+		}
+
+		if (info->type == GNOME_VFS_FILE_TYPE_SYMBOLIC_LINK) {
+			gnome_vfs_file_info_clear (info);
+			if (!brasero_utils_get_symlink_target (uri,
+							       info,
+							       data->flags)) {
+				/* since we checked for the existence of the file
+				 * an error means a looping symbolic link */
+				if (info->symlink_name)
+					result->result = GNOME_VFS_ERROR_LOOP;
+			}
+		}
+		result->info = info;
 	}
 
 	data->uris = NULL;
@@ -443,6 +579,7 @@ brasero_vfs_info_thread (BraseroAsyncTaskManager *manager,
 gboolean
 brasero_vfs_get_info (BraseroVFS *self,
 		      GList *uris,
+		      gboolean check_parent_sym,
 		      GnomeVFSFileInfoOptions flags,
 		      BraseroVFSDataID id,
 		      gpointer callback_data)
@@ -457,14 +594,13 @@ brasero_vfs_get_info (BraseroVFS *self,
 
 	data = g_new0 (BraseroInfoData, 1);
 	data->flags = flags;
+	data->check_parent_sym = check_parent_sym;
 
 	for (iter = uris; iter; iter = iter->next) {
-		GnomeVFSURI *vfsuri;
 		gchar *uri;
 
 		uri = iter->data;
-		vfsuri = gnome_vfs_uri_new (uri);
-		data->uris = g_slist_prepend (data->uris, vfsuri);
+		data->uris = g_slist_prepend (data->uris, g_strdup (uri));
 	}
 
 	BRASERO_CTX_TASK_DATA (ctx) = data;
@@ -528,9 +664,8 @@ brasero_vfs_load_result (BraseroAsyncTaskManager *manager,
 		BraseroInfoResult *result;
 
 		result = iter->data;
-
 		BRASERO_CTX_LOAD_CB (ctx,
-				     GNOME_VFS_OK,
+				     result->result,
 				     result->uri,
 				     result->info);
 	}
@@ -553,7 +688,6 @@ brasero_vfs_load_thread (BraseroAsyncTaskManager *manager,
 					data->root,
 					data->flags);
 	data->result = res;
-
 	if (res != GNOME_VFS_OK)
 		return;
 
@@ -561,6 +695,7 @@ brasero_vfs_load_thread (BraseroAsyncTaskManager *manager,
 	while (gnome_vfs_directory_read_next (handle, info) == GNOME_VFS_OK) {
 		BraseroInfoResult *result;
 		gchar *name;
+		gchar *uri;
 
 		if (ctx->cancelled)
 			break;
@@ -576,9 +711,24 @@ brasero_vfs_load_thread (BraseroAsyncTaskManager *manager,
 		data->results = g_slist_prepend (data->results, result);
 
 		name = gnome_vfs_escape_string (info->name);
-		result->uri = g_build_filename (data->root, name, NULL);
+		uri = g_build_filename (data->root, name, NULL);
 		g_free (name);
 
+		/* special case for symlinks */
+		result->result = GNOME_VFS_OK;
+		if (info->type == GNOME_VFS_FILE_TYPE_SYMBOLIC_LINK) {
+			gnome_vfs_file_info_clear (info);
+			if (!brasero_utils_get_symlink_target (uri,
+							       info,
+							       data->flags)) {
+				/* since we checked for the existence of the file
+				 * an error means a looping symbolic link */
+				if (info->symlink_name)
+					result->result = GNOME_VFS_ERROR_LOOP;
+			}
+		}
+
+		result->uri = uri;
 		result->info = info;
 
 		info = gnome_vfs_file_info_new ();
@@ -624,102 +774,82 @@ brasero_vfs_load_directory (BraseroVFS *self,
 static gboolean
 brasero_vfs_process_metadata (gpointer callback_data)
 {
-	BraseroMetadata *metadata;
+	BraseroMetadataFlag flags = BRASERO_METADATA_FLAG_NONE;
 	BraseroVFS *self = BRASERO_VFS (callback_data);
+	BraseroVFSMetadataTask *item;
+	GSList *iter;
 
 	if (!self->priv->unprocessed_meta) {
 		self->priv->meta_process_id = 0;
 		return FALSE;
 	}
 
-	/* to avoid too much overhead we only fetch
-	 * MAX_CONCURENT_META metadata files at a time */
-	if (g_slist_length (self->priv->processing_meta) >= MAX_CONCURENT_META) {
+	if (!self->priv->metadatas) {
+		/* Once one metadata object is free, this function will be
+		 * called again */
 		self->priv->meta_process_id = 0;
 		return FALSE;
 	}
 
-	metadata = self->priv->unprocessed_meta->data;
-	self->priv->unprocessed_meta = g_slist_remove (self->priv->unprocessed_meta, metadata);
-	self->priv->processing_meta = g_slist_prepend (self->priv->processing_meta, metadata);
+	item = self->priv->unprocessed_meta->data;
+	item->metadata = self->priv->metadatas->data;
+	self->priv->metadatas = g_slist_remove (self->priv->metadatas,
+						item->metadata);
 
-	brasero_metadata_get_async (metadata, FALSE);
+	BRASERO_BURN_LOG ("starting analysis of %s", item->uri);
+
+	self->priv->unprocessed_meta = g_slist_remove (self->priv->unprocessed_meta, item);
+	self->priv->processing_meta = g_slist_prepend (self->priv->processing_meta, item);
+
+	/* merge the flags of all contexts. NOTE: since the search for silences
+	 * implies and forces a thorough/long search, no need to care about fast
+	 * flag */
+	for (iter = item->ctx; iter; iter = iter->next) {
+		BraseroVFSTaskCtx *ctx;
+		BraseroMetadataData *data;
+
+		ctx = iter->data;
+		data = BRASERO_CTX_TASK_DATA (ctx);
+		flags |= data->flags;
+	}
+
+	brasero_metadata_get_info_async (item->metadata,
+					 item->uri,
+					 flags);
 	return TRUE;
 }
 
 static void
-brasero_vfs_metadata_processed (BraseroVFS *self,
-				BraseroMetadata *metadata)
+brasero_vfs_metadata_task_free (BraseroVFS *self,
+				BraseroVFSMetadataTask *item)
 {
-	if (!metadata)
+	if (!item)
 		return;
 
-	self->priv->processing_meta = g_slist_remove (self->priv->processing_meta,
-						      metadata);
-	if (!self->priv->meta_process_id
-	&&  g_slist_length (self->priv->processing_meta) < MAX_CONCURENT_META)
-		self->priv->meta_process_id = g_idle_add (brasero_vfs_process_metadata,
-							  self);
-
-	if (!metadata->has_audio && !metadata->has_video)
+	if (item->ctx)
 		return;
 
-	if (g_queue_find (self->priv->meta_buffer, metadata))
-		return;
+	/* there are no more context waiting, remove from the queue */
+	self->priv->unprocessed_meta = g_slist_remove (self->priv->unprocessed_meta, item);
+	self->priv->processing_meta = g_slist_remove (self->priv->processing_meta, item);
+	g_free (item->uri);
 
-	g_object_ref (metadata);
-	g_queue_push_head (self->priv->meta_buffer, metadata);
+	/* if there is a metadata (if it is processing) then:
+	 * cancel operation, put metadata in metadatas queue
+	 * and schedule a brasero_vfs_process_metadata */
+	if (item->metadata) {
+		brasero_metadata_cancel (item->metadata);
+		self->priv->metadatas = g_slist_prepend (self->priv->metadatas, item->metadata);
 
-	if (g_queue_get_length (self->priv->meta_buffer) > MAX_BUFFERED_META) {
-		metadata = g_queue_pop_tail (self->priv->meta_buffer);
-		g_object_unref (metadata);
-	}
-}
-
-static gboolean
-brasero_vfs_metadata_unref_cb (gpointer callback_data)
-{
-	BraseroMetadata *metadata = BRASERO_METADATA (callback_data);
-
-	brasero_metadata_cancel (metadata);
-	g_object_unref (metadata);
-
-	return FALSE;
-}
-
-static void
-brasero_vfs_metadata_refcount_changed (gpointer data,
-				       GObject *object,
-				       gboolean is_last_ref)
-{
-	BraseroVFS *self = BRASERO_VFS (data);
-
-	if (is_last_ref) {
-		self->priv->processing_meta = g_slist_remove (self->priv->processing_meta,
-							      object);
-		self->priv->unprocessed_meta = g_slist_remove (self->priv->unprocessed_meta,
-							       object);
-
-		g_object_ref (object);
-		g_object_remove_toggle_ref (object,
-					    brasero_vfs_metadata_refcount_changed,
-				            self);
-
-		/* NOTE: to work around a bug in Gstreamer (appearing with ogg)
-		 * we delay the unreffing */
-		g_timeout_add (STOP_META_TIMEOUT,
-			       brasero_vfs_metadata_unref_cb,
-			       object);
-
-		if (!self->priv->meta_process_id
-		&&  g_slist_length (self->priv->processing_meta) < MAX_CONCURENT_META)
-			self->priv->meta_process_id = g_idle_add (brasero_vfs_process_metadata,
-								  self);
+		if (!self->priv->meta_process_id && !self->priv->processing_meta)
+			self->priv->meta_process_id = g_idle_add (brasero_vfs_process_metadata, self);
 	}
+	g_free (item);
 }
 
 static void
 brasero_vfs_metadata_result_free (BraseroVFS *self,
+				  BraseroVFSTaskCtx *ctx,
 				  BraseroMetadataResult *result,
 				  gboolean cancelled)
 {
@@ -728,43 +858,46 @@ brasero_vfs_metadata_result_free (BraseroVFS *self,
 		result->info = NULL;
 	}
 
-	if (result->signal)
-		g_signal_handler_disconnect (result->metadata, result->signal);
-
-	if (result->metadata) {
-		g_object_unref (result->metadata);
-		result->metadata = NULL;
+	if (result->item) {
+		result->item->ctx = g_slist_remove (result->item->ctx, ctx);
+		brasero_vfs_metadata_task_free (self, result->item);
 	}
 
 	g_free (result);
 }
 
-static void
-brasero_vfs_metadata_completed_cb (BraseroMetadata *metadata,
-				   GError *error,
-				   gpointer user_data)
+static gint
+brasero_vfs_metadata_lookup_buffer (gconstpointer a, gconstpointer b)
 {
-	BraseroVFSTaskCtx *ctx = user_data;
-	BraseroMetadataResult *result;
-	BraseroMetadataData *data;
-	GSList *iter;
+	const BraseroMetadataInfo *metadata = a;
+	const gchar *uri = b;
 
-	brasero_vfs_metadata_processed (BRASERO_CTX_SELF (ctx),
-					metadata);
+	return strcmp (uri, metadata->uri);
+}
+
+static void
+brasero_vfs_metadata_ctx_completed (BraseroVFSTaskCtx *ctx,
+				    BraseroMetadataInfo *info)
+{
+	GSList *iter;
+	BraseroMetadataData *data;
+	BraseroMetadataResult *result;
 
 	data = BRASERO_CTX_TASK_DATA (ctx);
 
+	result = NULL;
 	for (iter = data->results; iter; iter = iter->next) {
 		result = iter->data;
 
-		if (result->metadata == metadata)
+		if (result->item
+		&& !strcmp (result->item->uri, info->uri))
 			break;
 
 		result = NULL;
 	}
 
 	if (!result) {
-		g_warning ("No corresponding BraseroMetadataResult found.\n");
+		BRASERO_BURN_LOG ("no corresponding BraseroMetadataResult found");
 		brasero_vfs_task_ctx_free (ctx, FALSE);
 		return;
 	}
@@ -777,23 +910,81 @@ brasero_vfs_metadata_completed_cb (BraseroMetadata *metadata,
 	 * is to look at has_audio/video */
 	BRASERO_CTX_META_CB (ctx,
 			     GNOME_VFS_OK, /* the result is OK since we searched for metadata */
-			     metadata->uri,
+			     info->uri,
 			     result->info,
-			     metadata);
+			     info);
 
-	brasero_vfs_metadata_result_free (BRASERO_CTX_SELF (ctx), result, FALSE);
+	brasero_vfs_metadata_result_free (BRASERO_CTX_SELF (ctx),
+					  ctx,
+					  result,
+					  FALSE);
 
 	if (!data->results || !data->refcount)
 		brasero_vfs_task_ctx_free (ctx, FALSE);
 }
 
-static gint
-brasero_vfs_metadata_lookup_buffer (gconstpointer a, gconstpointer b)
+static void
+brasero_vfs_metadata_completed_cb (BraseroMetadata *metadata,
+				   GError *error,
+				   gpointer user_data)
 {
-	const BraseroMetadata *metadata = a;
-	const gchar *uri = b;
+	BraseroVFS *self = BRASERO_VFS (user_data);
+	BraseroVFSMetadataTask *item = NULL;
+	BraseroMetadataInfo *info;
+	GSList *iter;
 
-	return strcmp (uri, metadata->uri);
+	info = g_new0 (BraseroMetadataInfo, 1);
+	brasero_metadata_set_info (metadata, info);
+
+	BRASERO_BURN_LOG ("analysis finished for %s", info->uri);
+
+	/* removed the currently processing metadata from the queue and possibly
+	 * add it to the buffer in case it is required again */
+	for (iter = self->priv->processing_meta; iter; iter = iter->next) {
+		item = iter->data;
+		if (!strcmp (item->uri, info->uri))
+			break;
+
+		item = NULL;
+	}
+
+	if (item) {
+		GSList *next;
+
+		self->priv->processing_meta = g_slist_remove (self->priv->processing_meta, item);
+
+		/* process results */
+		for (iter = item->ctx; iter; iter = next) {
+			BraseroVFSTaskCtx *ctx;
+
+			ctx = iter->data;
+			next = iter->next;
+			brasero_vfs_metadata_ctx_completed (ctx, info);
+	}
+
+		/* NOTE: no need to free item here it is freed once all contexts
+		 * have been removed from its queue by the above function */
+	}
+	else
+		BRASERO_BURN_LOG ("couldn't find waiting item in queue (cancelled?)");
+
+	/* remove it and check for the next file to process */
+	if (!self->priv->meta_process_id && !self->priv->processing_meta)
+		self->priv->meta_process_id = g_idle_add (brasero_vfs_process_metadata, self);
+
+	/* see if info is already in cache buffer */
+	if (g_queue_find_custom (self->priv->meta_buffer, info->uri, brasero_vfs_metadata_lookup_buffer))
+		return;
+
+	if (!info->has_audio && !info->has_video)
+		return;
+
+	g_queue_push_head (self->priv->meta_buffer, info);
+
+	if (g_queue_get_length (self->priv->meta_buffer) > MAX_BUFFERED_META) {
+		info = g_queue_pop_tail (self->priv->meta_buffer);
+		brasero_metadata_info_free (info);
+	}
 }
 
 static gboolean
@@ -802,68 +993,55 @@ brasero_vfs_metadata_ctx_new (BraseroVFS *self,
 			      const gchar *uri,
 			      GnomeVFSFileInfo *info)
 {
-	BraseroMetadata *metadata = NULL;
-	BraseroMetadataData *data;
+	BraseroVFSMetadataTask *item = NULL;
 	BraseroMetadataResult *result;
+	BraseroMetadataData *data;
 	GSList *iter;
 
-	/* see if one of the metadata in the queue (unprocessed or processing)
-	 * is not about the same uri: if so ref the metadata and add a CB */
+	/* make sure the URI is going to be processed. see if self isn't already
+	 * processing the same URI or if the URI is not in the unprocessed queue
+	 */
 	for (iter = self->priv->unprocessed_meta; iter; iter = iter->next) {
-		metadata = iter->data;
-		if (!strcmp (metadata->uri, uri)) {
-			g_object_ref (metadata);
+		item = iter->data;
+		if (!strcmp (item->uri, uri))
 			break;
-		}
 
-		metadata = NULL;
+		item = NULL;
 	}
 
-	for (iter = self->priv->processing_meta; iter && !metadata; iter = iter->next) {
-		metadata = iter->data;
-		if (!strcmp (metadata->uri, uri))
-			g_object_ref (metadata);
-		else
-			metadata = NULL;
+	/* see if it is currently being processed */
+	for (iter = self->priv->processing_meta; iter; iter = iter->next) {
+		item = iter->data;
+		if (!strcmp (item->uri, uri))
+			break;
+
+		item = NULL;
 	}
+	
+	if (!item) {
+		/* there was nothing to process. Add it */
+		item = g_new0 (BraseroVFSMetadataTask, 1);
+		item->uri = g_strdup (uri);
 
-	if (!metadata) {
-		metadata = brasero_metadata_new (uri);
+		if (!self->priv->meta_process_id && !self->priv->processing_meta)
+			self->priv->meta_process_id = g_idle_add (brasero_vfs_process_metadata, self);
 
-		if (!metadata)
-			return FALSE;
-
-		g_object_add_toggle_ref (G_OBJECT (metadata),
-					 brasero_vfs_metadata_refcount_changed,
-					 self);
-
-		if (!self->priv->meta_process_id
-		&&  g_slist_length (self->priv->processing_meta) <= MAX_CONCURENT_META)
-			self->priv->meta_process_id = g_idle_add (brasero_vfs_process_metadata,
-								  self);
-
-		self->priv->unprocessed_meta = g_slist_append (self->priv->unprocessed_meta, metadata);
+		self->priv->unprocessed_meta = g_slist_append (self->priv->unprocessed_meta, item);
 	}
 
 	/* add a reference to the parent */
 	data = BRASERO_CTX_TASK_DATA (ctx);
 	data->refcount ++;
 
-	/* we start with a refcount of 1 since we're going to call a function */
 	result = g_new0 (BraseroMetadataResult, 1);
 	data->results = g_slist_prepend (data->results, result);
 
 	/* keep info for later: add a reference */
 	gnome_vfs_file_info_ref (info);
 	result->info = info;
+	result->item = item;
 
-	/* connect to the completed signal and wait for metadata object
-	 * to be started and to complete its job */
-	result->metadata = metadata;
-	result->signal = g_signal_connect (G_OBJECT (metadata),
-					   "completed",
-					   G_CALLBACK (brasero_vfs_metadata_completed_cb),
-					   ctx);
+	item->ctx = g_slist_prepend (item->ctx, ctx);
 	return TRUE;
 }
 
@@ -889,7 +1067,10 @@ brasero_vfs_metadata_destroy (GObject *object,
 		BraseroMetadataResult *result;
 
 		result = iter->data;
-		brasero_vfs_metadata_result_free (self, result, cancelled);
+		brasero_vfs_metadata_result_free (self,
+						  ctx,
+						  result,
+						  cancelled);
 	}
 
 	g_slist_free (data->results);
@@ -976,7 +1157,7 @@ brasero_vfs_metadata_result (BraseroVFS *self,
 					    brasero_vfs_metadata_lookup_buffer);
 
 		if (node) {
-			BraseroMetadata *metadata;
+			BraseroMetadataInfo *metadata;
 
 			metadata = node->data;
 			BRASERO_CTX_META_CB (ctx,
@@ -993,7 +1174,8 @@ brasero_vfs_metadata_result (BraseroVFS *self,
 gboolean
 brasero_vfs_get_metadata (BraseroVFS *self,
 			  GList *uris,
-			  GnomeVFSFileInfoOptions flags,
+			  GnomeVFSFileInfoOptions vfs_flags,
+			  BraseroMetadataFlag meta_flags,
 			  gboolean recursive,
 			  BraseroVFSDataID id,
 			  gpointer callback_data)
@@ -1010,6 +1192,7 @@ brasero_vfs_get_metadata (BraseroVFS *self,
 	data = g_new0 (BraseroMetadataData, 1);
 	data->refcount = 1;
 	data->recursive = recursive;
+	data->flags = meta_flags;
 
 	BRASERO_CTX_TASK_DATA (ctx) = data;
 	BRASERO_CTX_SET_TASK_METHOD (self, ctx, BRASERO_TASK_TYPE_METADATA);
@@ -1017,7 +1200,8 @@ brasero_vfs_get_metadata (BraseroVFS *self,
 	BRASERO_CTX_SET_METATASK (ctx);
 	result = brasero_vfs_get_info (self,
 				       uris,
-				       flags|
+				       FALSE,
+				       vfs_flags|
 				       GNOME_VFS_FILE_INFO_FOLLOW_LINKS|
 				       GNOME_VFS_FILE_INFO_GET_MIME_TYPE|
 				       GNOME_VFS_FILE_INFO_FORCE_SLOW_MIME_TYPE,
@@ -1078,7 +1262,7 @@ brasero_vfs_count_result_audio (BraseroVFS *self,
 			        GnomeVFSResult result,
 			        const gchar *uri,
 			        GnomeVFSFileInfo *info,
-			        BraseroMetadata *metadata,
+			        BraseroMetadataInfo *metadata,
 			        gpointer user_data)
 {
 	BraseroVFSTaskCtx *ctx = user_data;
@@ -1137,7 +1321,6 @@ brasero_vfs_count_result_data (BraseroVFS *self,
 	data = BRASERO_CTX_TASK_DATA (ctx);
 
 	data->files_num ++;
-
 	if (result != GNOME_VFS_OK) {
 		data->invalid_num ++;
 		return;
@@ -1147,7 +1330,7 @@ brasero_vfs_count_result_data (BraseroVFS *self,
 		data->refcount ++;
 		brasero_vfs_load_directory (BRASERO_CTX_SELF (ctx),
 					    uri,
-					    GNOME_VFS_FILE_INFO_FOLLOW_LINKS,
+					    GNOME_VFS_FILE_INFO_DEFAULT,//GNOME_VFS_FILE_INFO_FOLLOW_LINKS,
 					    BRASERO_TASK_TYPE_COUNT_SUBTASK_DATA,
 					    ctx);
 	}
@@ -1181,16 +1364,18 @@ brasero_vfs_get_count (BraseroVFS *self,
 	if (audio)
 		return brasero_vfs_get_metadata (self,
 						 uris,
-						 GNOME_VFS_FILE_INFO_FOLLOW_LINKS|
+						// GNOME_VFS_FILE_INFO_FOLLOW_LINKS|
 						 GNOME_VFS_FILE_INFO_FORCE_SLOW_MIME_TYPE|
 						 GNOME_VFS_FILE_INFO_GET_MIME_TYPE,
+						 0,
 						 FALSE,
 						 BRASERO_TASK_TYPE_COUNT_SUBTASK_AUDIO,
 						 ctx);
 	else
 		return brasero_vfs_get_info (self,
 					     uris,
-					     GNOME_VFS_FILE_INFO_FOLLOW_LINKS,
+					     TRUE,
+					     GNOME_VFS_FILE_INFO_DEFAULT,//GNOME_VFS_FILE_INFO_FOLLOW_LINKS,
 					     BRASERO_TASK_TYPE_COUNT_SUBTASK_DATA,
 					     ctx);
 }
@@ -1220,6 +1405,11 @@ brasero_vfs_playlist_destroy (GObject *object,
 		data->uri = NULL;
 	}
 
+	if (data->title) {
+		g_free (data->title);
+		data->title = NULL;
+	}
+
 	if (data->parser) {
 		g_object_unref (data->parser);
 		data->parser = NULL;
@@ -1251,16 +1441,18 @@ brasero_vfs_playlist_subtask_result (BraseroVFS *self,
 				     GnomeVFSResult result,
 				     const gchar *uri,
 				     GnomeVFSFileInfo *info,
-				     BraseroMetadata *metadata,
+				     BraseroMetadataInfo *metadata,
 				     gpointer user_data)
 {
 	BraseroVFSTaskCtx *ctx = user_data;
+	BraseroVFSPlaylistData *data;
 
-	BRASERO_CTX_PLAYLIST_CB (ctx,
-				 result,
-				 uri,
-				 info,
-				 metadata);
+	data = BRASERO_CTX_TASK_DATA (ctx);
+	BRASERO_CTX_META_CB (ctx,
+			     result,
+			     uri,
+			     info,
+			     metadata);
 }
 
 static void
@@ -1277,19 +1469,26 @@ brasero_vfs_playlist_result (BraseroAsyncTaskManager *manager,
 	data->parser = NULL;
 
 	if (data->res != TOTEM_PL_PARSER_RESULT_SUCCESS) {
-		BRASERO_CTX_PLAYLIST_CB (ctx,
-					 GNOME_VFS_ERROR_GENERIC,
-					 data->uri,
-					 NULL,
-					 NULL);
+		BRASERO_CTX_META_CB (ctx,
+				     GNOME_VFS_ERROR_GENERIC,
+				     data->uri,
+				     NULL,
+				     NULL);
 		return;
 	}
+	
+	BRASERO_CTX_META_CB (ctx,
+			     GNOME_VFS_OK,
+			     data->title,
+			     NULL,
+			     NULL);
 
 	BRASERO_CTX_SET_METATASK (ctx);
 	data->uris = g_list_reverse (data->uris);
 	brasero_vfs_get_metadata (self,
 				  data->uris,
 				  data->flags,
+				  BRASERO_METADATA_FLAG_NONE,
 				  FALSE,
 				  BRASERO_TASK_TYPE_PLAYLIST_SUBTASK,
 				  ctx);				       
@@ -1305,6 +1504,17 @@ brasero_vfs_add_playlist_entry_cb (TotemPlParser *parser,
 	data->uris = g_list_prepend (data->uris, g_strdup (uri));
 }
 
+static void
+brasero_vfs_start_end_playlist_cb (TotemPlParser *parser,
+				   const gchar *title,
+				   BraseroVFSPlaylistData *data)
+{
+	if (!title)
+		return;
+
+	if (!data->title)
+		data->title = g_strdup (title);}
+
 static void
 brasero_vfs_playlist_thread (BraseroAsyncTaskManager *manager,
 			     gpointer callback_data)
@@ -1313,6 +1523,14 @@ brasero_vfs_playlist_thread (BraseroAsyncTaskManager *manager,
 	BraseroVFSTaskCtx *ctx = callback_data;
 
 	data = BRASERO_CTX_TASK_DATA (ctx);
+	g_signal_connect (G_OBJECT (data->parser),
+			  "playlist-start",
+			  G_CALLBACK (brasero_vfs_start_end_playlist_cb),
+			  data);
+	g_signal_connect (G_OBJECT (data->parser),
+			  "playlist-end",
+			  G_CALLBACK (brasero_vfs_start_end_playlist_cb),
+			  data);
 	g_signal_connect (G_OBJECT (data->parser),
 			  "entry",
 			  G_CALLBACK (brasero_vfs_add_playlist_entry_cb),
@@ -1438,6 +1656,48 @@ brasero_vfs_stop_all (BraseroVFS *self)
 	}
 }
 
+struct _BraseroVFSTaskCompareData {
+	BraseroVFSCompareFunc func;
+	BraseroVFSDataType *type;
+	gpointer user_data;
+};
+typedef struct _BraseroVFSTaskCompareData BraseroVFSTaskCompareData;
+
+static gboolean
+brasero_vfs_compare_unprocessed_task (BraseroAsyncTaskManager *manager,
+				      gpointer task,
+				      gpointer callback_data)
+{
+	BraseroVFSTaskCtx *ctx = task;
+	BraseroVFSTaskCompareData *data = callback_data;
+
+	if (ctx->user_method != data->type)
+		return FALSE;
+
+	return data->func (ctx->user_data, data->user_data);
+}
+
+gboolean
+brasero_vfs_find_urgent (BraseroVFS *self,
+			 BraseroVFSDataID id,
+			 BraseroVFSCompareFunc func,
+			 gpointer user_data)
+{
+	BraseroVFSTaskCompareData callback_data;
+
+	g_return_val_if_fail (self != NULL, FALSE);
+	g_return_val_if_fail (func != NULL, FALSE);
+
+	callback_data.func = func;
+	callback_data.user_data = user_data;
+	callback_data.type = g_hash_table_lookup (self->priv->types,
+						  GINT_TO_POINTER (id));
+
+	return brasero_async_task_manager_find_urgent_task (BRASERO_ASYNC_TASK_MANAGER (self),
+							    brasero_vfs_compare_unprocessed_task,
+							    &callback_data);
+}
+
 /**
  * used to register new data type
  */
@@ -1481,8 +1741,21 @@ brasero_vfs_register_data_type (BraseroVFS *self,
 static void
 brasero_vfs_init (BraseroVFS *obj)
 {
+	int i;
+
 	obj->priv = g_new0 (BraseroVFSPrivate, 1);
 
+	for (i = 0; i < MAX_CONCURENT_META; i ++) {
+		BraseroMetadata *metadata;
+
+		metadata = brasero_metadata_new ();
+		g_signal_connect (G_OBJECT (metadata),
+				  "completed",
+				  G_CALLBACK (brasero_vfs_metadata_completed_cb),
+				  obj);
+		obj->priv->metadatas = g_slist_prepend (obj->priv->metadatas, metadata);
+	}
+
 	obj->priv->meta_buffer = g_queue_new ();
 
 	obj->priv->types = g_hash_table_new_full (g_direct_hash,
@@ -1565,10 +1838,10 @@ brasero_vfs_finalize (GObject *object)
 	cobj = BRASERO_VFS (object);
 
 	if (cobj->priv->meta_buffer) {
-		BraseroMetadata *metadata;
+		BraseroMetadataInfo *metadata;
 
 		while ((metadata = g_queue_pop_head (cobj->priv->meta_buffer)) != NULL)
-			g_object_unref (metadata);
+			brasero_metadata_info_free (metadata);
 
 		g_queue_free (cobj->priv->meta_buffer);
 		cobj->priv->meta_buffer = NULL;
@@ -1581,6 +1854,18 @@ brasero_vfs_finalize (GObject *object)
 		cobj->priv->types = NULL;
 	}
 
+	g_slist_foreach (cobj->priv->unprocessed_meta, (GFunc) brasero_vfs_metadata_task_free, NULL);
+	g_slist_free (cobj->priv->unprocessed_meta);
+	cobj->priv->unprocessed_meta = NULL;
+
+	g_slist_foreach (cobj->priv->processing_meta, (GFunc) brasero_vfs_metadata_task_free, NULL);
+	g_slist_free (cobj->priv->processing_meta);
+	cobj->priv->processing_meta = NULL;
+
+	g_slist_foreach (cobj->priv->metadatas, (GFunc) g_object_unref, NULL);
+	g_slist_free (cobj->priv->metadatas);
+	cobj->priv->metadatas = NULL;
+
 	g_free (cobj->priv);
 
 	G_OBJECT_CLASS (parent_class)->finalize (object);
