@@ -100,7 +100,7 @@ struct _BraseroMediumPrivate
 	gint64 block_size;
 
 	guint64 next_wr_add;
-	BraseroMediumInfo info;
+	BraseroMedia info;
 	NautilusBurnDrive * drive;
 };
 
@@ -109,7 +109,6 @@ struct _BraseroMediumPrivate
 enum
 {
 	PROP_0,
-
 	PROP_DRIVE
 };
 
@@ -133,7 +132,7 @@ brasero_medium_get_icon (BraseroMedium *medium)
 	return priv->icon;
 }
 
-BraseroMediumInfo
+BraseroMedia
 brasero_medium_get_status (BraseroMedium *medium)
 {
 	BraseroMediumPrivate *priv;
@@ -151,8 +150,10 @@ brasero_medium_get_tracks (BraseroMedium *medium)
 	return g_slist_copy (priv->tracks);
 }
 
-gint64
-brasero_medium_get_last_data_track_address (BraseroMedium *medium)
+gboolean
+brasero_medium_get_last_data_track_address (BraseroMedium *medium,
+					    gint64 *byte,
+					    gint64 *sector)
 {
 	GSList *iter;
 	BraseroMediumPrivate *priv;
@@ -168,9 +169,159 @@ brasero_medium_get_last_data_track_address (BraseroMedium *medium)
 			track = current;
 	}
 
-	if (!track)
-		return -1;
-	return track->start;
+	if (!track) {
+		if (byte)
+			*byte = -1;
+		if (sector)
+			*sector = -1;
+		return FALSE;
+	}
+
+	if (byte)
+		*byte = track->start * priv->block_size;
+
+	if (sector)
+		*sector = track->start;
+
+	return TRUE;
+}
+
+gboolean
+brasero_medium_get_last_data_track_space (BraseroMedium *medium,
+					  gint64 *size,
+					  gint64 *blocks)
+{
+	GSList *iter;
+	BraseroMediumPrivate *priv;
+	BraseroMediumTrack *track = NULL;
+
+	priv = BRASERO_MEDIUM_PRIVATE (medium);
+
+	for (iter = priv->tracks; iter; iter = iter->next) {
+		BraseroMediumTrack *current;
+
+		current = iter->data;
+		if (current->type & BRASERO_MEDIUM_TRACK_DATA)
+			track = current;
+	}
+
+	if (!track) {
+		if (size)
+			*size = -1;
+		if (blocks)
+			*blocks = -1;
+		return FALSE;
+	}
+
+	if (size)
+		*size = track->blocks_num * priv->block_size;
+	if (blocks)
+		*blocks = track->blocks_num;
+
+	return TRUE;
+}
+
+guint
+brasero_medium_get_track_num (BraseroMedium *medium)
+{
+	guint retval = 0;
+	GSList *iter;
+	BraseroMediumPrivate *priv;
+
+	priv = BRASERO_MEDIUM_PRIVATE (medium);
+	for (iter = priv->tracks; iter; iter = iter->next) {
+		BraseroMediumTrack *current;
+
+		current = iter->data;
+		if (current->type & BRASERO_MEDIUM_TRACK_LEADOUT)
+			break;
+
+		retval ++;
+	}
+
+	return retval;
+}
+
+static BraseroMediumTrack *
+brasero_medium_get_track (BraseroMedium *medium,
+			  guint num)
+{
+	guint i = 1;
+	GSList *iter;
+	BraseroMediumPrivate *priv;
+
+	priv = BRASERO_MEDIUM_PRIVATE (medium);
+
+	for (iter = priv->tracks; iter; iter = iter->next) {
+		BraseroMediumTrack *current;
+
+		current = iter->data;
+		if (current->type == BRASERO_MEDIUM_TRACK_LEADOUT)
+			break;
+
+		if (i == num)
+			return current;
+
+		i++;
+	}
+
+	return NULL;
+}
+
+gboolean
+brasero_medium_get_track_space (BraseroMedium *medium,
+				guint num,
+				gint64 *size,
+				gint64 *blocks)
+{
+	BraseroMediumPrivate *priv;
+	BraseroMediumTrack *track;
+
+	priv = BRASERO_MEDIUM_PRIVATE (medium);
+
+	track = brasero_medium_get_track (medium, num);
+	if (!track) {
+		if (size)
+			*size = -1;
+		if (blocks)
+			*blocks = -1;
+		return FALSE;
+	}
+
+	if (size)
+		*size = track->blocks_num * priv->block_size;
+	if (blocks)
+		*blocks = track->blocks_num;
+
+	return TRUE;
+}
+
+gboolean
+brasero_medium_get_track_address (BraseroMedium *medium,
+				  guint num,
+				  gint64 *byte,
+				  gint64 *sector)
+{
+	BraseroMediumPrivate *priv;
+	BraseroMediumTrack *track;
+
+	priv = BRASERO_MEDIUM_PRIVATE (medium);
+
+	track = brasero_medium_get_track (medium, num);
+	if (!track) {
+		if (byte)
+			*byte = -1;
+		if (sector)
+			*sector = -1;
+		return FALSE;
+	}
+
+	if (byte)
+		*byte = track->start * priv->block_size;
+	if (sector)
+		*sector = track->start;
+
+	return TRUE;	
 }
 
 gint64
@@ -182,13 +333,13 @@ brasero_medium_get_next_writable_address (BraseroMedium *medium)
 	return priv->next_wr_add;
 }
 
-gint
+gint64
 brasero_medium_get_max_write_speed (BraseroMedium *medium)
 {
 	BraseroMediumPrivate *priv;
 
 	priv = BRASERO_MEDIUM_PRIVATE (medium);
-	return priv->max_wrt;
+	return priv->max_wrt * 1024;
 }
 
 /**
@@ -226,6 +377,17 @@ brasero_medium_get_data_size (BraseroMedium *medium,
 
 	priv = BRASERO_MEDIUM_PRIVATE (medium);
 
+	if (!priv->tracks) {
+		/* that's probably because it wasn't possible to retrieve info */
+		if (size)
+			*size = 0;
+
+		if (blocks)
+			*blocks = 0;
+
+		return;
+	}
+
 	for (iter = priv->tracks; iter; iter = iter->next) {
 		BraseroMediumTrack *tmp;
 
@@ -237,10 +399,10 @@ brasero_medium_get_data_size (BraseroMedium *medium,
 	}
 
 	if (size)
-		*size = track ? (track->start + track->blocks_num) * priv->block_size: -1;
+		*size = track ? (track->start + track->blocks_num) * priv->block_size: 0;
 
 	if (blocks)
-		*blocks = track ? track->start + track->blocks_num: -1;
+		*blocks = track ? track->start + track->blocks_num: 0;
 }
 
 void
@@ -254,6 +416,28 @@ brasero_medium_get_free_space (BraseroMedium *medium,
 
 	priv = BRASERO_MEDIUM_PRIVATE (medium);
 
+	if (!priv->tracks) {
+		/* that's probably because it wasn't possible to retrieve info.
+		 * maybe it also happens with unformatted DVD+RW */
+
+		if (priv->info & BRASERO_MEDIUM_CLOSED) {
+			if (size)
+				*size = 0;
+
+			if (blocks)
+				*blocks = 0;
+		}
+		else {
+			if (size)
+				*size = priv->block_num * priv->block_size;
+
+			if (blocks)
+				*blocks = priv->block_num;
+		}
+
+		return;
+	}
+
 	for (iter = priv->tracks; iter; iter = iter->next) {
 		BraseroMediumTrack *tmp;
 
@@ -264,11 +448,29 @@ brasero_medium_get_free_space (BraseroMedium *medium,
 		}
 	}
 
-	if (size)
-		*size = track ? track->blocks_num * priv->block_size: -1;
+	if (size) {
+		if (!track) {
+			/* No leadout was found so the disc is probably closed:
+			 * no free space left. */
+			*size = 0;
+		}
+		else if (track->blocks_num <= 0)
+			*size = (priv->block_num - track->start) * priv->block_size;
+		else
+			*size = track->blocks_num * priv->block_size;
+	}
 
-	if (blocks)
-		*blocks = track ? track->blocks_num: -1;
+	if (blocks) {
+		if (!track) {
+			/* No leadout was found so the disc is probably closed:
+			 * no free space left. */
+			*blocks = 0;
+		}
+		else if (track->blocks_num <= 0)
+			*blocks = priv->block_num - track->blocks_num;
+		else
+			*blocks = track->blocks_num;
+	}
 }
 
 void
@@ -287,7 +489,7 @@ brasero_medium_get_capacity (BraseroMedium *medium,
 		if (blocks)
 			*blocks = priv->block_num;
 	}
-	else  if (!(priv->info & BRASERO_MEDIUM_WRITABLE))
+	else  if (priv->info & BRASERO_MEDIUM_CLOSED)
 		brasero_medium_get_data_size (medium, size, blocks);
 	else
 		brasero_medium_get_free_space (medium, size, blocks);
@@ -308,6 +510,8 @@ brasero_medium_get_capacity_CD_RW (BraseroMedium *self,
 
 	priv = BRASERO_MEDIUM_PRIVATE (self);
 
+	BRASERO_BURN_LOG ("Retrieving capacity from atip");
+
 	result = brasero_mmc1_read_atip (fd,
 					 &atip_data,
 					 sizeof (atip_data),
@@ -322,6 +526,10 @@ brasero_medium_get_capacity_CD_RW (BraseroMedium *self,
 					      atip_data.desc->leadout_sec,
 					      atip_data.desc->leadout_frame);
 
+	BRASERO_BURN_LOG ("Format capacity %lli %lli",
+			  priv->block_num,
+			  priv->block_size);
+
 	return BRASERO_BURN_OK;
 }
 
@@ -336,6 +544,8 @@ brasero_medium_get_capacity_DVD_RW (BraseroMedium *self,
 	BraseroScsiResult result;
 	gint size;
 
+	BRASERO_BURN_LOG ("Retrieving format capacity");
+
 	priv = BRASERO_MEDIUM_PRIVATE (self);
 	result = brasero_mmc2_read_format_capacities (fd,
 						      &hdr,
@@ -366,6 +576,10 @@ brasero_medium_get_capacity_DVD_RW (BraseroMedium *self,
 				if (desc->format_type == BRASERO_SCSI_DVDRW_PLUS) {
 					priv->block_num = BRASERO_GET_32 (desc->blocks_num);
 					priv->block_size = BRASERO_GET_24 (desc->type_param);
+
+					/* that can happen */
+					if (!priv->block_size)
+						priv->block_size = 2048;
 					break;
 				}
 			}
@@ -381,6 +595,10 @@ brasero_medium_get_capacity_DVD_RW (BraseroMedium *self,
 		priv->block_size = BRASERO_GET_24 (current->block_size);
 	}
 
+	BRASERO_BURN_LOG ("Format capacity %lli %lli",
+			  priv->block_num,
+			  priv->block_size);
+
 	g_free (hdr);
 	return BRASERO_BURN_OK;
 }
@@ -424,6 +642,8 @@ brasero_medium_get_speed_mmc3 (BraseroMedium *self,
 	BraseroScsiWrtSpdDesc *desc;
 	BraseroScsiGetPerfData *wrt_perf = NULL;
 
+	BRASERO_BURN_LOG ("Retrieving speed (Get Performance)");
+
 	/* NOTE: this only work if there is RT streaming feature with
 	 * wspd bit set to 1. At least an MMC3 drive. */
 	priv = BRASERO_MEDIUM_PRIVATE (self);
@@ -471,7 +691,7 @@ end:
 	 * function but don't report any speed. So if our top speed is 0 then
 	 * use the other way to get the speed. It was a Teac */
 	if (!priv->max_wrt)
-		return BRASERO_BURN_RETRY;
+		return BRASERO_BURN_ERR;
 
 	return BRASERO_BURN_OK;
 }
@@ -491,6 +711,8 @@ brasero_medium_get_page_2A_write_speed_desc (BraseroMedium *self,
 	gint max_num;
 	int size = 0;
 
+	BRASERO_BURN_LOG ("Retrieving speed (2A speeds)");
+
 	priv = BRASERO_MEDIUM_PRIVATE (self);
 	result = brasero_spc1_mode_sense_get_page (fd,
 						   BRASERO_SPC_PAGE_STATUS,
@@ -506,6 +728,7 @@ brasero_medium_get_page_2A_write_speed_desc (BraseroMedium *self,
 
 	page_2A = (BraseroScsiStatusPage *) &data->page;
 
+	/* FIXME: the following is not necessarily true */
 	if (size < sizeof (BraseroScsiStatusPage)) {
 		g_free (data);
 
@@ -554,6 +777,8 @@ brasero_medium_get_page_2A_max_speed (BraseroMedium *self,
 	BraseroScsiResult result;
 	int size = 0;
 
+	BRASERO_BURN_LOG ("Retrieving speed (2A max)");
+
 	priv = BRASERO_MEDIUM_PRIVATE (self);
 
 	result = brasero_spc1_mode_sense_get_page (fd,
@@ -590,11 +815,12 @@ brasero_medium_get_medium_type (BraseroMedium *self,
 				BraseroScsiErrCode *code)
 {
 	BraseroScsiGetConfigHdr *hdr = NULL;
-	BraseroScsiRTStreamDesc *stream;
 	BraseroMediumPrivate *priv;
 	BraseroScsiResult result;
 	int size;
 
+	BRASERO_BURN_LOG ("Retrieving media profile");
+
 	priv = BRASERO_MEDIUM_PRIVATE (self);
 	result = brasero_mmc2_get_configuration_feature (fd,
 							 BRASERO_SCSI_FEAT_REAL_TIME_STREAM,
@@ -626,7 +852,7 @@ brasero_medium_get_medium_type (BraseroMedium *self,
 
 	switch (BRASERO_GET_16 (hdr->current_profile)) {
 	case BRASERO_SCSI_PROF_CDROM:
-		priv->info = BRASERO_MEDIUM_CD;
+		priv->info = BRASERO_MEDIUM_CDROM;
 		priv->type = types [1];
 		priv->icon = icons [1];
 		break;
@@ -644,7 +870,7 @@ brasero_medium_get_medium_type (BraseroMedium *self,
 		break;
 
 	case BRASERO_SCSI_PROF_DVD_ROM:
-		priv->info = BRASERO_MEDIUM_DVD;
+		priv->info = BRASERO_MEDIUM_DVD_ROM;
 		priv->type = types [4];
 		priv->icon = icons [4];
 		break;
@@ -679,6 +905,7 @@ brasero_medium_get_medium_type (BraseroMedium *self,
 		priv->icon = icons [7];
 		break;
 
+	/* WARNING: these types are recognized, no more */
 	case BRASERO_SCSI_PROF_DVD_R_PLUS_DL:
 		priv->info = BRASERO_MEDIUM_DVDR_PLUS_DL;
 		priv->type = types [9];
@@ -745,40 +972,39 @@ brasero_medium_get_medium_type (BraseroMedium *self,
 	case BRASERO_SCSI_PROF_HD_DVD_R:
 	case BRASERO_SCSI_PROF_HD_DVD_RAM:
 		priv->info = BRASERO_MEDIUM_UNSUPPORTED;
+		priv->icon = icons [0];
 		g_free (hdr);
 		return BRASERO_BURN_NOT_SUPPORTED;
 	}
 
-	BRASERO_BURN_LOG ("medium is %d", priv->info);
-
-	/* see how we should get the speeds */
-	if (hdr->desc->add_len != sizeof (BraseroScsiRTStreamDesc)) {
-		g_free (hdr);
-
-		BRASERO_BURN_LOG ("wrong size for Stream descriptor");
-		return BRASERO_BURN_ERR;
-	}
-
 	/* try all SCSI functions to get write/read speeds in order */
-	stream = (BraseroScsiRTStreamDesc *) hdr->desc->data;
-	if (stream->wrt_spd) {
-		result = brasero_medium_get_speed_mmc3 (self, fd, code);
-		if (result != BRASERO_BURN_RETRY)
-			goto end;
+	if (hdr->desc->add_len >= sizeof (BraseroScsiRTStreamDesc)) {
+		BraseroScsiRTStreamDesc *stream;
+
+		/* means it's at least an MMC3 drive */
+		stream = (BraseroScsiRTStreamDesc *) hdr->desc->data;
+		if (stream->wrt_spd) {
+			result = brasero_medium_get_speed_mmc3 (self, fd, code);
+			if (result == BRASERO_BURN_OK)
+				goto end;
+		}
+
+		if (stream->mp2a) {
+			result = brasero_medium_get_page_2A_write_speed_desc (self, fd, code);
+			if (result == BRASERO_BURN_OK)
+				goto end;
+		}
 	}
 
-	if (stream->mp2a)
-		result = brasero_medium_get_page_2A_write_speed_desc (self, fd, code);
-	else
-		result = brasero_medium_get_page_2A_max_speed (self, fd, code);
-
+	/* fallback for speeds */
+	result = brasero_medium_get_page_2A_max_speed (self, fd, code);
 
 end:
 
 	g_free (hdr);
 
 	if (result != BRASERO_BURN_OK)
-		return BRASERO_BURN_ERR;
+		return result;
 
 	return BRASERO_BURN_OK;
 }
@@ -794,6 +1020,8 @@ brasero_medium_get_css_feature (BraseroMedium *self,
 	int size;
 
 	priv = BRASERO_MEDIUM_PRIVATE (self);
+
+	BRASERO_BURN_LOG ("Testing for Css encrypted media");
 	result = brasero_mmc2_get_configuration_feature (fd,
 							 BRASERO_SCSI_FEAT_DVD_CSS,
 							 &hdr,
@@ -806,14 +1034,16 @@ brasero_medium_get_css_feature (BraseroMedium *self,
 		return BRASERO_BURN_ERR;
 	}
 
-	if (hdr->desc->add_len != sizeof (BraseroScsiDVDCssDesc)) {
+	if (hdr->desc->add_len < sizeof (BraseroScsiDVDCssDesc)) {
 		g_free (hdr);
-		return BRASERO_BURN_ERR;
+		return BRASERO_BURN_OK;
 	}
 
 	/* here we just need to see if this feature is current or not */
-	if (hdr->desc->current)
+	if (hdr->desc->current) {
 		priv->info |= BRASERO_MEDIUM_PROTECTED;
+		BRASERO_BURN_LOG ("media is Css protected");
+	}
 
 	g_free (hdr);
 	return BRASERO_BURN_OK;
@@ -837,6 +1067,7 @@ brasero_medium_track_volume_size (BraseroMedium *self,
 {
 	BraseroMediumPrivate *priv;
 	BraseroBurnResult res;
+	GError *error = NULL;
 	gint64 nb_blocks;
 
 	if (!track)
@@ -857,8 +1088,15 @@ brasero_medium_track_volume_size (BraseroMedium *self,
 					  track->start,
 					  &nb_blocks,
 					  NULL);
-	if (!res)
+	if (!res) {
+		BRASERO_BURN_LOG ("Failed to retrieve the volume size: %s",
+				  error && error->message ? 
+				  error->message:"unknown error");
+
+		if (error)
+			g_error_free (error);
 		return BRASERO_BURN_ERR;
+	}
 
 	track->blocks_num = nb_blocks;
 	return BRASERO_BURN_OK;
@@ -874,13 +1112,26 @@ brasero_medium_track_get_info (BraseroMedium *self,
 	BraseroScsiTrackInfo track_info;
 	BraseroMediumPrivate *priv;
 	BraseroScsiResult result;
+	int size;
+
+	BRASERO_BURN_LOG ("Retrieving track information for %i", track_num);
 
 	priv = BRASERO_MEDIUM_PRIVATE (self);
 
+	/* at this point we know the type of the disc that's why we set the 
+	 * size according to this type. That may help to avoid outrange address
+	 * errors. */
+	if (BRASERO_MEDIUM_IS (priv->info, BRASERO_MEDIUM_DL|BRASERO_MEDIUM_WRITABLE))
+		size = 48;
+	else if (BRASERO_MEDIUM_IS (priv->info, BRASERO_MEDIUM_PLUS|BRASERO_MEDIUM_WRITABLE))
+		size = 40;
+	else
+		size = 36;
+
 	result = brasero_mmc1_read_track_info (fd,
 					       track_num,
 					       &track_info,
-					       sizeof (BraseroScsiTrackInfo),
+					       &size,
 					       code);
 
 	if (result != BRASERO_SCSI_OK) {
@@ -898,15 +1149,57 @@ brasero_medium_track_get_info (BraseroMedium *self,
 	 * use the old value.
 	 * That's important for checksuming to have a perfect account of the 
 	 * data size. */
-	if (track->blocks_num == 300)
+	if (track->blocks_num == 300) {
+		BRASERO_BURN_LOG ("300 sectors size. Checking for real size");
 		brasero_medium_track_volume_size (self, track, fd);
+	}
 
 	if (track_info.next_wrt_address_valid)
 		priv->next_wr_add = BRASERO_GET_32 (track_info.next_wrt_address);
 
+	BRASERO_BURN_LOG ("Track %i: type = %i start = %llu size = %llu",
+			  track_num,
+			  track->type,
+			  track->start,
+			  track->blocks_num);
+
 	return BRASERO_BURN_OK;
 }
 
+static void
+brasero_medium_add_DVD_plus_RW_leadout (BraseroMedium *self,
+					gint32 start)
+{
+	BraseroMediumTrack *leadout;
+	BraseroMediumPrivate *priv;
+
+	priv = BRASERO_MEDIUM_PRIVATE (self);
+
+	leadout = g_new0 (BraseroMediumTrack, 1);
+	priv->tracks = g_slist_append (priv->tracks, leadout);
+
+	leadout->start = start;
+	leadout->type = BRASERO_MEDIUM_TRACK_LEADOUT;
+
+	/* we fabricate the leadout here. We don't really need one in 
+	 * fact since it is always at the last sector whatever the
+	 * amount of data written. So we need in fact to read the file
+	 * system and get the last sector from it. Hopefully it won't be
+	 * buggy */
+	priv->next_wr_add = 0;
+
+	leadout->blocks_num = priv->block_num;
+	if (g_slist_length (priv->tracks) > 1) {
+		BraseroMediumTrack *track;
+
+		track = priv->tracks->data;
+		leadout->blocks_num -= ((track->blocks_num > 300) ? track->blocks_num : 300);
+	}
+	BRASERO_BURN_LOG ("Adding fabricated leadout start = %llu length = %llu",
+			  leadout->start,
+			  leadout->blocks_num);
+}
+
 static BraseroBurnResult
 brasero_medium_get_sessions_info (BraseroMedium *self,
 				  int fd,
@@ -918,6 +1211,8 @@ brasero_medium_get_sessions_info (BraseroMedium *self,
 	BraseroMediumPrivate *priv;
 	BraseroScsiFormattedTocData *toc = NULL;
 
+	BRASERO_BURN_LOG ("Reading toc");
+
 	priv = BRASERO_MEDIUM_PRIVATE (self);
 	result = brasero_mmc1_read_toc_formatted (fd,
 						  0,
@@ -934,6 +1229,8 @@ brasero_medium_get_sessions_info (BraseroMedium *self,
 	num = (size - sizeof (BraseroScsiFormattedTocData)) /
 	       sizeof (BraseroScsiTocDesc);
 
+	BRASERO_BURN_LOG ("%i track(s) found", num);
+
 	desc = toc->desc;
 	for (i = 0; i < num; i ++, desc ++) {
 		BraseroMediumTrack *track;
@@ -967,7 +1264,7 @@ brasero_medium_get_sessions_info (BraseroMedium *self,
 		}
 		else if (BRASERO_MEDIUM_IS (priv->info, BRASERO_MEDIUM_DVDRW_PLUS)
 		     ||  BRASERO_MEDIUM_IS (priv->info, BRASERO_MEDIUM_DVDRW_RESTRICTED)) {
-				BraseroBurnResult result;
+			BraseroBurnResult result;
 
 			/* a special case for these two kinds of media
 			 * which have only one track: the first. */
@@ -976,8 +1273,8 @@ brasero_medium_get_sessions_info (BraseroMedium *self,
 								   fd);
 			if (result == BRASERO_BURN_OK) {
 				track->type |= BRASERO_MEDIUM_TRACK_DATA;
-				priv->info |= BRASERO_MEDIUM_APPENDABLE|
-					      BRASERO_MEDIUM_HAS_DATA;
+				priv->info |= BRASERO_MEDIUM_HAS_DATA;
+
 				priv->next_wr_add = 0;
 
 				if (desc->control & BRASERO_SCSI_TRACK_DATA_INCREMENTAL)
@@ -988,6 +1285,7 @@ brasero_medium_get_sessions_info (BraseroMedium *self,
 				g_free (track);
 
 				priv->info |= BRASERO_MEDIUM_BLANK;
+				priv->info &= ~BRASERO_MEDIUM_CLOSED;
 			}
 		}
 		else {
@@ -999,12 +1297,19 @@ brasero_medium_get_sessions_info (BraseroMedium *self,
 		}
 	}
 
-	/* we shouldn't request info on leadout if the disc is closed */
-	if (priv->info & (BRASERO_MEDIUM_APPENDABLE|BRASERO_MEDIUM_BLANK)) {
+	/* put the tracks in the right order */
+	priv->tracks = g_slist_reverse (priv->tracks);
+
+	if (BRASERO_MEDIUM_IS (priv->info, BRASERO_MEDIUM_DVDRW_PLUS)
+	||  BRASERO_MEDIUM_IS (priv->info, BRASERO_MEDIUM_DVDRW_RESTRICTED))
+		brasero_medium_add_DVD_plus_RW_leadout (self, BRASERO_GET_32 (desc->track_start));
+	else if (!(priv->info & BRASERO_MEDIUM_CLOSED)) {
 		BraseroMediumTrack *track;
 
+		/* we shouldn't request info on leadout if the disc is closed
+		 * (except for DVD+/- (restricted) RW (see above) */
 		track = g_new0 (BraseroMediumTrack, 1);
-		priv->tracks = g_slist_prepend (priv->tracks, track);
+		priv->tracks = g_slist_append (priv->tracks, track);
 		track->start = BRASERO_GET_32 (desc->track_start);
 		track->type = BRASERO_MEDIUM_TRACK_LEADOUT;
 
@@ -1015,29 +1320,6 @@ brasero_medium_get_sessions_info (BraseroMedium *self,
 					       code);
 	}
 
-	/* put the tracks in the right order */
-	priv->tracks = g_slist_reverse (priv->tracks);
-
-	if (BRASERO_MEDIUM_IS (priv->info, BRASERO_MEDIUM_DVDRW_PLUS)
-	||  BRASERO_MEDIUM_IS (priv->info, BRASERO_MEDIUM_DVDRW_RESTRICTED)) {
-		GSList *node;
-		BraseroMediumTrack *leadout;
-
-		node = g_slist_last (priv->tracks);
-		leadout = node->data;
-
-		if (g_slist_length (priv->tracks) > 1) {
-			BraseroMediumTrack *track;
-
-			track = priv->tracks->data;
-			leadout->blocks_num -= track->blocks_num;
-		}
-		else {
-			leadout->blocks_num = leadout->start;
-			leadout->start = 0;
-		}
-	}
-
 	g_free (toc);
 
 	return BRASERO_BURN_OK;
@@ -1053,6 +1335,8 @@ brasero_medium_get_contents (BraseroMedium *self,
 	BraseroMediumPrivate *priv;
 	BraseroScsiDiscInfoStd *info = NULL;
 
+	BRASERO_BURN_LOG ("Retrieving media status");
+
 	priv = BRASERO_MEDIUM_PRIVATE (self);
 
 	result = brasero_mmc1_read_disc_information_std (fd,
@@ -1072,19 +1356,37 @@ brasero_medium_get_contents (BraseroMedium *self,
 	if (info->status == BRASERO_SCSI_DISC_EMPTY) {
 		BraseroMediumTrack *track;
 
+		BRASERO_BURN_LOG ("Empty media");
+
 		priv->info |= BRASERO_MEDIUM_BLANK;
 		priv->block_size = 2048;
 
-		track = g_new0 (BraseroMediumTrack, 1);
-		track->start = 0;
-		track->type = BRASERO_MEDIUM_TRACK_LEADOUT;
-		priv->tracks = g_slist_prepend (priv->tracks, track);
-		brasero_medium_track_get_info (self, track, 1, fd, code);
+		if (BRASERO_MEDIUM_IS (priv->info, BRASERO_MEDIUM_DVDRW_PLUS)
+		||  BRASERO_MEDIUM_IS (priv->info, BRASERO_MEDIUM_DVDRW_RESTRICTED))
+			brasero_medium_add_DVD_plus_RW_leadout (self, 0);
+		else {
+			track = g_new0 (BraseroMediumTrack, 1);
+			track->start = 0;
+			track->type = BRASERO_MEDIUM_TRACK_LEADOUT;
+			priv->tracks = g_slist_prepend (priv->tracks, track);
+			
+			brasero_medium_track_get_info (self,
+						       track,
+						       1,
+						       fd,
+						       code);
+		}
 		goto end;
 	}
 
-	if (info->status == BRASERO_SCSI_DISC_INCOMPLETE)
+	if (info->status == BRASERO_SCSI_DISC_INCOMPLETE) {
 		priv->info |= BRASERO_MEDIUM_APPENDABLE;
+		BRASERO_BURN_LOG ("Appendable media");
+	}
+	else if (info->status == BRASERO_SCSI_DISC_FINALIZED) {
+		priv->info |= BRASERO_MEDIUM_CLOSED;
+		BRASERO_BURN_LOG ("Closed media");
+	}
 
 	result = brasero_medium_get_sessions_info (self, fd, code);
 	if (result != BRASERO_BURN_OK)
@@ -1109,17 +1411,18 @@ brasero_medium_init_real (BraseroMedium *object, int fd)
 	if (result != BRASERO_BURN_OK)
 		return;
 
-	if (priv->info & BRASERO_MEDIUM_DVD) {
-		result = brasero_medium_get_css_feature (object, fd, &code);
-		if (result != BRASERO_BURN_OK)
-			return;
-	}
+	brasero_medium_get_capacity_by_type (object, fd, &code);
 
 	result = brasero_medium_get_contents (object, fd, &code);
 	if (result != BRASERO_BURN_OK)
 		return;
 
-	brasero_medium_get_capacity_by_type (object, fd, &code);
+	/* assume that css feature is only for DVD-ROM which might be wrong but
+	 * some drives wrongly reports that css is enabled for blank DVD+R/W */
+	if (BRASERO_MEDIUM_IS (priv->info, (BRASERO_MEDIUM_DVD|BRASERO_MEDIUM_ROM)))
+		brasero_medium_get_css_feature (object, fd, &code);
+
+	BRASERO_BURN_LOG_DISC_TYPE (priv->info, "media is ");
 }
 
 static gboolean
@@ -1153,6 +1456,8 @@ brasero_medium_retry_open (gpointer object)
 
 	BRASERO_BURN_LOG ("Open () succeeded\n");
 	priv->info = BRASERO_MEDIUM_NONE;
+	priv->icon = icons [0];
+
 	priv->retry_id = 0;
 
 	brasero_medium_init_real (self, fd);
@@ -1181,6 +1486,8 @@ brasero_medium_try_open (BraseroMedium *self)
 		||  errno == EBUSY) {
 			BRASERO_BURN_LOG ("Device busy");
 			priv->info = BRASERO_MEDIUM_BUSY;
+			priv->icon = icons [0];
+
 			priv->retry_id = g_timeout_add (1000,
 							brasero_medium_retry_open,
 							self);
